<?php

namespace Elementor;

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}


/**
 * Elementor tabs widget.
 *
 * Elementor widget that displays vertical or horizontal tabs with different
 * pieces of content.
 *
 * @since 1.0.0
 */
class Rebuilto_Elementor_Timeline_Slider extends Widget_Base {

    public function get_categories() {
        return array('rebuilto-addons');
    }

    /**
     * Get widget name.
     *
     * Retrieve tabs widget name.
     *
     * @return string Widget name.
     * @since 1.0.0
     * @access public
     *
     */
    public function get_name() {
        return 'rebuilto-timeline-slider';
    }

    /**
     * Get widget title.
     *
     * Retrieve tabs widget title.
     *
     * @return string Widget title.
     * @since 1.0.0
     * @access public
     *
     */
    public function get_title() {
        return esc_html__('Rebuilto Slider Timeline', 'rebuilto');
    }

    /**
     * Get widget icon.
     *
     * Retrieve tabs widget icon.
     *
     * @return string Widget icon.
     * @since 1.0.0
     * @access public
     *
     */
    public function get_icon() {
        return 'eicon-slider-3d';
    }

    public function get_script_depends() {
        return ['rebuilto-elementor-timeline-slider'];
    }

    /**
     * Register tabs widget controls.
     *
     * Adds different input fields to allow the user to change and customize the widget settings.
     *
     * @since 1.0.0
     * @access protected
     */
    protected function register_controls() {

        $templates = Plugin::instance()->templates_manager->get_source('local')->get_items();

        $options = [
            '0' => '— ' . esc_html__('Select', 'rebuilto') . ' —',
        ];

        foreach ($templates as $template) {
            $options[$template['template_id']] = $template['title'] . ' (' . $template['type'] . ')';
        }

        $this->start_controls_section(
            'section_slider',
            [
                'label' => esc_html__('Slider', 'rebuilto'),
            ]
        );

        $this->add_responsive_control(
            'slides_to_show',
            [
                'label'              => esc_html__('Slides to Show', 'rebuilto'),
                'type'               => Controls_Manager::NUMBER,
                'frontend_available' => true,
                'default'            => 3,
                'render_type'        => 'template',
                'selectors'          => [
                    '{{WRAPPER}} .main-swiper.swiper:not(.swiper-initialized) .swiper-slide' => 'width: calc(100% / {{VALUE}});',
                ],
            ]
        );

        $this->add_responsive_control(
            'timeline_spacing',
            [
                'label'     => esc_html__('Timeline Spacing', 'rebuilto'),
                'type'      => Controls_Manager::SLIDER,
                'range'     => [
                    'px' => [
                        'min' => 0,
                        'max' => 200,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .main-swiper' => 'margin-bottom: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_responsive_control(
            'timeline_width',
            [
                'label'     => esc_html__('Timeline Width', 'rebuilto'),
                'type'      => Controls_Manager::SLIDER,
                'default'        => [
                    'unit' => '%',
                ],
                'tablet_default' => [
                    'unit' => '%',
                ],
                'mobile_default' => [
                    'unit' => '%',
                ],
                'size_units'     => ['%', 'px'],
                'range'          => [
                    '%'  => [
                        'min' => 1,
                        'max' => 100,
                    ],
                    'px' => [
                        'min' => 1,
                        'max' => 1290,
                    ],
                ],
                'selectors' => [
                    '{{WRAPPER}} .wrapper-timeline-main' => 'width: {{SIZE}}{{UNIT}};',
                ],
            ]
        );

        $this->add_group_control(
            Group_Control_Typography::get_type(),
            [
                'name'     => 'time_typography',
                'selector' => '{{WRAPPER}} .main-swiper .timeline-title',
            ]
        );

        $repeater = new Repeater();

        $repeater->add_control(
            'title',
            [
                'label'       => esc_html__('Title', 'rebuilto'),
                'type'        => Controls_Manager::TEXT,
                'label_block' => true,
            ]
        );


        $repeater->add_control(
            'template',
            [
                'label'       => esc_html__('Choose Template', 'rebuilto'),
                'default'     => 0,
                'type'        => Controls_Manager::SELECT,
                'options'     => $options,
                'label_block' => true,
            ]
        );
        $this->add_control(
            'sliders',
            [
                'label'       => '',
                'type'        => Controls_Manager::REPEATER,
                'fields'      => $repeater->get_controls(),
                'title_field' => '{{{ title }}}',
            ]
        );

        $this->end_controls_section();


    }

    /**
     * Render tabs widget output on the frontend.
     *
     * Written in PHP and used to generate the final HTML.
     *
     * @since 1.0.0
     * @access protected
     */
    protected function render() {
        $settings = $this->get_settings_for_display();
        $this->add_render_attribute([
            'item'            => [
                'class' => 'swiper-slide',
            ],
            'main_wrapper'    => [
                'class' => 'rebuilto-swiper swiper main-swiper',
            ],
            'content_wrapper' => [
                'class' => 'rebuilto-swiper swiper content-swiper',
            ],
            'row'             => [
                'class' => 'swiper-wrapper',
            ],
        ]);

        $content_html = '';

        ?>
        <div class="wrapper-timeline-main">
            <div <?php $this->print_render_attribute_string('main_wrapper'); ?>>
                <div <?php $this->print_render_attribute_string('row'); ?>>
                    <?php foreach ($settings['sliders'] as $item) {
                        ?>
                        <div <?php $this->print_render_attribute_string('item'); // WPCS: XSS ok. ?>>
                            <div class="timeline-item">
                                <?php
                                echo '<span class="timeline-title">' . esc_html($item['title']) . '</span>';
                                ?>
                            </div>
                        </div>
                        <?php
                        ob_start();
                        ?>
                        <div <?php $this->print_render_attribute_string('item'); // WPCS: XSS ok. ?>>
                            <?php
                            echo Plugin::instance()->frontend->get_builder_content_for_display($item['template']);
                            ?>
                        </div>
                        <?php
                        $content_html .= ob_get_clean();
                    } ?>
                </div>
            </div>
        </div>
        <div <?php $this->print_render_attribute_string('content_wrapper'); ?>>
            <div <?php $this->print_render_attribute_string('row'); ?>>
                <?php printf('%s', $content_html); ?>
            </div>
        </div>

        <?php
    }
}

$widgets_manager->register(new Rebuilto_Elementor_Timeline_Slider());
